%GeomEst.m
%  Script to Estimate  Fault Geometry. Modified for IZMIT earthquake 08/99

%Poisson's ratio
        nu = 0.25;

%Update fault geometry and station coordinates in local coords
        UpdateGeom_orig

if length(d_gps) > 0
        xy_gps = llh2localxy(llh_gps, origin);
end

%save the starting model
       !/usr/bin/rm temp.gmtlin
	[nf, nel] = size(geom);
        for i=1:nf
           displot_gmt( geom(i,:), dis_geom(i,:) )
        end
	!/usr/bin/mv temp.gmtlin start.gmtlin
%
% Read constraints
	[xlb, xub] = GetBounds(nf);
%
%Starting Value
	x0 = geom2x(geom, origin, xlb, xub);
%
% Set Options for Optimization program

options(1) = 1;
options(2) = 1.0e-2;
options(3) = 1.0e-2;
%options(5) = 1;                        % use Gauss-newton
%options(9) = 1;
%options(14) = 100;

%
[x, options] = constr('Constr_WRSS',x0, options, -1.0*ones(size(xlb)), ones(size(xub)), [ ], origin, xy_gps, nu, d_gps, Icov, geom, xlb, xub);


disp(' number of function evals'), options(10)

[geom_est, dis_geom_est] = x2geom(x, origin, xlb, xub);
disp(' estimated model: len, wid, depth, dip, strike, DeltaE, DeltaN')
        for i=1:nf
x(i*7-6:i*7)
 dis_geom_est(i,:)
        end
disp(' estimated model: wid, depth, dip, Lat-Lon 1end, Lat-Lon 2end')
geom_est
dis_geom = [dis_geom_est, geom(:,8:10)];
geom     = [geom_est, dis_geom(:,8:10)];

% CREATE DESIGN MATRIX

% GPS DATA
if length(d_gps) > 0
        xy_gps = llh2localxy(llh_gps, origin);

        if nf > 0
                G_dis = MakeGgps(dis_geom, xy_gps, nu);
        else
                G_dis = [];
        end
        if nv > 0
                G_vol = MakeGvolgps(vol_geom, xy_gps, nu);
        else
                G_vol = [];
        end


        G = [G_dis, G_vol];
else
        G = [];
end

% Clear Plot Files
% invert and estimate slip
        Ginv = pinv(W*G);
        slip = Ginv*W*d;
        cov_slip = Ginv*Ginv';
        sig_slip = sqrt(diag(cov_slip));
        dhat = G*slip;
        r = W*(d - dhat);
%        r_raw = (d - dhat);
 
% Output results
        PrintSlip(slip, sig_slip, nf, 0);
%        PrintResids(r, d_gps, d_lev, d_tilt, G);
	PrintResids(r, d_gps, G);

% PRINT OUT MOMENT AND MOMENT MAGNITUDE
                clear Mo
                clear Mw
mmslip = slip/1000;
for i=1:nf;

                Mo(i) = 1000 * dis_geom(i,1) * 1000 * dis_geom(i,2) * abs(sqrt(mmslip(i*3-2)^2 + mmslip(i*3-1)^2)) * 30E9;
                Mw(i) = 2/3*log10(Mo(i)) - 6.0;
                disp(['   Fault Number ', num2str(i)]);
                fprintf(1,'Moment %4.2e',  Mo(i) ); disp(' (Nm)'); disp('  ')
                fprintf(1,'Mw %8.4f',  Mw(i) ); disp('  ')
        end
                Mo_total = sum(Mo);
                Mw_total = 2/3*log10(Mo_total) - 6.0;
                disp('   Total Geodetic Moment ');
                disp(' _______________________ ');
                fprintf(1,'Moment %4.2e',  Mo_total ); disp(' (Nm)'); disp('  ')
                fprintf(1,'Mw %8.4f',  Mw_total ); disp(' Geodetic Moment Magnitude ')

% END PRINT OUT MOMENT AND MOMENT MAGNITUDE

% Write out volume change if opening is involved
for i=1:nf;
        if dis_geom(i,10)~=0
                clear volchg
                volchg(i) = dis_geom(i,1) * dis_geom(i,2) * slip(i*3)/1000;
                disp(['   Fault Number ', num2str(i)]);
                fprintf(1,'Volume change %8.4f',  volchg(i) ); disp(' (km^3)'); disp('  ')
        end
end
% Clear Plot Files
        !/usr/bin/rm temp.gmtlin
        !/usr/bin/rm obs.gmtvec
        !/usr/bin/rm mod.gmtvec
        !/usr/bin/rm res.gmtvec

% IF horizontal data exists -- plot  vectors
 
if length(d_gps) > 0
% This is for a 3D covariance Matrix!
%        [a,b,az] = ErrlpsGMT( cov3d2cov2d(cov_gps)  );
%        nvec = length(d_gps)/3;

% This is for a 2D covariance Matrix!
        [a,b,az] = ErrlpsGMT(cov_gps);
        nvec = length(d_gps)/2;

        outmatrix1 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)', vel(1,1:nvec)', ...
                vel(2,1:nvec)', a',b',az'];

        % plot predicted vectors
% This is for 3D
%        u_pre = zeros(3,nvec);
%        for i = 1:nvec
%                u_pre(:,i)  = dhat( 3*(i-1)+1 : 3*(i-1)+3 );
%        end
% This is for 2D
        u_pre = zeros(2,nvec);
        for i = 1:nvec
                u_pre(:,i)  = dhat( 2*(i-1)+1 : 2*(i-1)+2 );
        end
        null = zeros(size(u_pre(1,:)))';
        outmatrix2 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)',  u_pre(1,:)', ...
                  u_pre(2,:)', null, null, null];

% plot residual vectors
        res =   (vel(:,1:nvec)) - u_pre(:,:) ;

        outmatrix3 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)',  res(1,:)',   res(2,:)', null,null,null];

        save obs.gmtvec outmatrix1 -ascii
        save mod.gmtvec outmatrix2 -ascii
        save res.gmtvec outmatrix3 -ascii
end

% Plot Fault Outlines and Volume Sources in GMT plot
 
        if nf > 0
                for i=1:nf
                 displot_gmt( geom(i,:), dis_geom(i,:) )
                end
        end
        
        if nv >0
                 volplot_gmt( vgeom );
        end

% Run Script
        if get(cb_plt, 'value') == 1
                !composite.map
        end


% IF Leveling data exists plot observed and predicted
%IF LEVELING DATA EXISTS, PLOT GMT FILES
if length(d_lev) > 0

        % plot observed level data
        figure
        errorbar (Len,d_lev, sigma_lev, sigma_lev, 'y+')
        hold on
	title('Fit to Leveling Data')
	xlabel('Distance Along Leveling Route')
	ylabel('Elevation Change')

        % plot modeled level data
        plot (Len,dhat(length(d_gps)+1:length(d_gps)+length(d_lev)), 'g*')
        hold off;
% GMT PLOTS
        !/usr/bin/rm levelobs.gmtvec
        !/usr/bin/rm levelmod.gmtvec 
        !/usr/bin/rm levelres.gmtvec
        clear outmatrix4 
        clear outmatrix5
        clear outmatrix6
        outmatrix4 = [llh_lev(2,:)', llh_lev(1,:)', d_lev];
        dhat_lev = dhat(length(d_gps)+1:length(d_gps)+length(d_lev));
        outmatrix5 = [llh_lev(2,:)', llh_lev(1,:)', dhat_lev];
        res_lev = d_lev - dhat_lev;
        outmatrix6 = [llh_lev(2,:)', llh_lev(1,:)', res_lev];

        save levelobs.gmtvec outmatrix4 -ascii
        save levelmod.gmtvec outmatrix5 -ascii
        save levelres.gmtvec outmatrix6 -ascii

end
 
% IF TILT DATA EXISTS -- PLOT TILT DATA IN GMT

%if length(d_tilt) > 0
%        clear outmatrix1
%        clear outmatrix2
%        clear nvec
%        clear a
%        clear b
%        clear az
%        nsta = length(d_tilt)/2
%plot observed vectors
%        a(1) = sqrt(cov_tilt(1,1));
%        b(1) = sqrt(cov_tilt(2,2));
%        az(1) = 0;
%for i = 4:2:(nsta*2)
%        a(i/2) = sqrt(cov_tilt(i-1,i-1));
%        b(i/2) = sqrt(cov_tilt(i,i));
%        az(i/2) = 0;
%end
%        nvec = length(d_tilt_v);
%        outmatrix1 = [llh_tilt(2,1:nvec)', llh_tilt(1,1:nvec)', d_tilt_v(1,:)', ...
%                d_tilt_v(2,:)',a',b',az'];
% 
%        % plot predicted vectors
%        u_pre = zeros(2,nvec);
%        i_tilt = length(d_gps)+length(d_lev) +1
%        dhat_tilt = dhat(length(d_gps)+length(d_lev)+1:length(dhat));
%for i = 1: nvec
%                u_pre(:,i )  = dhat_tilt( 2*(i-1)+1 : 2*(i-1)+2 );
%end
%        null = zeros(size(u_pre(1,:)))';
%        outmatrix2 = [llh_tilt(2,1:nvec)', llh_tilt(1,1:nvec)', u_pre(1,:)', ...
%                 u_pre(2,:)', null, null, null];
%
%        save tiltobs.gmtvec outmatrix1 -ascii
%        save tiltmod.gmtvec outmatrix2 -ascii
%end

%make_quiver

disp('Done')
 
