        function [vel, cov, llh] = GetGMTdata(filename)
%
% Read velocity file in GMT format
% Output:
%	vel 	- vector of 3D velocities
%	cov	- covariance matrix of velocities
%	llh	- latitude longitude height 

% CLEAR ARRAYS
clear llh
clear vel
clear cov

filename = filename(1,:);
eval(['fopen ',filename]);
fid = ans;
if ans == -1
disp('Could not read file'); disp(filename)
else

%       read the data
        [A, count] = fscanf(fid, '%g %g %g %g %g %g %g %g', [8, inf]);
        A = A';

	nsta =length(A) 
	nvec=nsta-1

% LOAD LLH
        llh = A(1:nsta,1:2)';
% LOAD DISPLACMENTS RELATIVE TO LAST SITE IN METER
vel(2,:) = A(1:nvec,3)'-A(nsta,3);
vel(1,:) = A(1:nvec,4)'-A(nsta,4);
vel(3,:) =  A(1:nvec,5)'-A(nsta,5);
a        =  A(1:nvec,7)';
b        =  A(1:nvec,6)';
c        =  A(1:nvec,8)';

% FOR COSEISMIC mm to m
vel = vel/1000;
a = a/1000;
b = b/1000;
c = c/1000;
% 3D COV
	cov = zeros(3*nvec,3*nvec);
	cov(1,1) = (a(1))^2;
	cov(2,2) = (b(1))^2;
	cov(3,3) = (c(1))^2;
for i = 4:3:(nvec*3)
	cov(i,i) = (a(i/3 + 2/3))^2;
	cov(i+1,i+1) = (b(i/3 + 2/3))^2;
	cov(i+2,i+2) = (c(i/3 + 2/3))^2;
end

% APPLY GAUSSIAN NOISE TO THE DATA
%x=randn(3,nvec);
%
%for i = 1:nvec
%   vel_add(1,i) = x(1,i)/2.4*a(i);
%   vel_add(2,i) = x(2,i)/2.4*b(i);
%   vel_add(3,i) = x(3,i)/2.4*c(i);
%end
%   vel = vel + vel_add;
%
%	disp('WARNING! WARNING! APPLIED GAUSSIAN NOISE TO DATA!')
%
%Scale Sigmas by 1.5;
%cov = 1.5*1.5 * cov;
%	disp('Scaled Sigmas by factor of 1.5; cov by 1.5^2 !')
	disp(' Data File Read: 3D Relative Displacements, NO Correlations!')
	fclose(ans);
end

