        function [vel, cov, llh] = GetGMTdata(filename)
%
% Read velocity file in GMT format
% Input GMT Se format only!!:
%Modified March 10, 1999.
%       E, N velocities vel (assume cm/yr-- convert to m/yr below
%	with multiplication by 0.01).
%       esig, nsig, correlation (assume cm/yr for esig,nsig-- 
%	convert to m/yr below
%	with multiplication by 0.01).
% Output:
%	vel 	- vector of velocities
%	cov	- covariance matrix of velocities (2D only, or fill 3rd D)
%	llh	- latitude longitude height

filename = filename(1,:);
eval(['fopen ',filename]);
fid = ans;
if ans == -1
	disp('Could not read file'); disp(filename)
else

% Load data file
%       read the data
        [A, count] = fscanf(fid, '%g %g %g %g %g %g %g', [7, inf]);
        nsta = count/7

filename = A';
nsta1 =length(filename(:,1)'); 
nvec=nsta-1;
% load station coordinates
clear llh
clear vel
clear cov
llh = zeros(3,nsta);
% 3D
% vel = zeros(3,nvec);
% 2D
vel = zeros(2,nvec);
llh(2,:) = filename(:,1)';
llh(1,:) = filename(:,2)';

% load observed vectors
vel(1,:) = filename(1:nvec,3)';
vel(2,:) = filename(1:nvec,4)';
esig     = filename(1:nvec,5)';
nsig     = filename(1:nvec,6)';
encorr     = filename(1:nvec,7)';

% Compute 2D covariance matrix
%All other covariance elements would be zero. This is 2D of course,

% Initialize 2D cov matrix
	cov = zeros(2*nvec,2*nvec);

% Initialize 3D cov matrix
%	cov = zeros(3*nvec,3*nvec);

for i = 1:nvec
% 2D case (from Jeff Freymueller)
      k = 1 + 2*(i-1);

% "3D" case fill height cov as 1 m
%      k = 1 + 3*(i-1);

      cov(k,k) = esig(i)*esig(i);
      cov(k+1,k+1) = nsig(i)*nsig(i);
%      cov(k+2,k+2) = 100;
      cov(k,k+1) = esig(i)*nsig(i)*encorr(i);
      cov(k+1,k) = esig(i)*nsig(i)*encorr(i);
end
	disp(' Data File Read')
	fclose(ans);
end

