function [LAP]=MakeLapF(nsegs,ndeps)

%MakeLapF makes a Laplacian smoothing filter for 3 slip components. 
%        Usage: [LAP]=MakeLapF(nsegs,ndeps)

% Note: The model vector must be presorted in depth and along-strike to
%	use this filter. So, the model vector has elements m(d0,as0),
%	m(d0,as1),m(d0,as2),.....m(d1,as0),m(d1,as1),m(d1,as2)....m(df,asf). 

nmod=nsegs*ndeps;

LAP=zeros(nmod*3,nmod*3);
for i=1:ndeps,
  for j=1:nsegs,
    indm=3*nsegs*(i-1)+3*(j-1)+1;
    inda=3*nsegs*(i-2)+3*(j-1)+1;
    indb=3*nsegs*i+3*(j-1)+1;
    indl=3*nsegs*(i-1)+3*(j-1)-2;
    indr=3*nsegs*(i-1)+3*(j-1)+4;
    for m=1:nmod*3,
      if m == indm
        LAP(indm,indm)=-1;
        LAP(indm+1,indm+1)=-1;
        LAP(indm+2,indm+2)=-1;
      elseif m == inda
	if i==1 & j==1
          LAP(indm,inda)=0.0;
          LAP(indm+1,inda+1)=0.0;
          LAP(indm+2,inda+2)=0.0;
	elseif i==1 & j>1 & j<nsegs 
          LAP(indm,inda)=0.0;
          LAP(indm+1,inda+1)=0.0;
          LAP(indm+2,inda+2)=0.0;
	elseif i==1 & j==nsegs
          LAP(indm,inda)=0.0;
          LAP(indm+1,inda+1)=0.0;
          LAP(indm+2,inda+2)=0.0;
        elseif j==1 & i>1 & i<ndeps
          LAP(indm,inda)=1./3;
          LAP(indm+1,inda+1)=1./3;
          LAP(indm+2,inda+2)=1./3;
        elseif j==1 & i==ndeps
          LAP(indm,inda)=0.5;
          LAP(indm+1,inda+1)=0.5;
          LAP(indm+2,inda+2)=0.5;
	elseif i==ndeps & j>1 & j<nsegs
	  LAP(indm,inda)=1./3;
          LAP(indm+1,inda+1)=1./3;
          LAP(indm+2,inda+2)=1./3;
	elseif i==ndeps & j==nsegs
	  LAP(indm,inda)=0.5;
          LAP(indm+1,inda+1)=0.5;
          LAP(indm+2,inda+2)=0.5;
	elseif j==nsegs & i>1 & i<ndeps
	  LAP(indm,inda)=1./3;
          LAP(indm+1,inda+1)=1./3;
          LAP(indm+2,inda+2)=1./3;
	else
	  LAP(indm,inda)=0.25;
          LAP(indm+1,inda+1)=0.25;
          LAP(indm+2,inda+2)=0.25;
	end
      elseif m == indb
	if i==1 & j==1
          LAP(indm,indb)=0.5;
          LAP(indm+1,indb+1)=0.5;
          LAP(indm+2,indb+2)=0.5;
	elseif i==1 & j>1 & j<nsegs 
          LAP(indm,indb)=1./3;
          LAP(indm+1,indb+1)=1./3;
          LAP(indm+2,indb+2)=1./3;
	elseif i==1 & j==nsegs
          LAP(indm,indb)=0.5;
          LAP(indm+1,indb+1)=0.5;
          LAP(indm+2,indb+2)=0.5;
        elseif j==1 & i>1 & i<ndeps
          LAP(indm,indb)=1./3;
          LAP(indm+1,indb+1)=1./3;
          LAP(indm+2,indb+2)=1./3;
        elseif j==1 & i==ndeps
          LAP(indm,indb)=0.0;
          LAP(indm+1,indb+1)=0.0;
          LAP(indm+2,indb+2)=0.0;
	elseif i==ndeps & j>1 & j<nsegs
	  LAP(indm,indb)=0.0;
          LAP(indm+1,indb+1)=0.0;
          LAP(indm+2,indb+2)=0.0;
	elseif i==ndeps & j==nsegs
	  LAP(indm,indb)=0.0;
          LAP(indm+1,indb+1)=0.0;
          LAP(indm+2,indb+2)=0.0;
	elseif j==nsegs & i>1 & i<ndeps
	  LAP(indm,indb)=1./3;
          LAP(indm+1,indb+1)=1./3;
          LAP(indm+2,indb+2)=1./3;
	else
	  LAP(indm,indb)=0.25;
          LAP(indm+1,indb+1)=0.25;
          LAP(indm+2,indb+2)=0.25;
	end
      elseif m == indl
	if i==1 & j==1
          LAP(indm,indl)=0.0;
          LAP(indm+1,indl+1)=0.0;
          LAP(indm+2,indl+2)=0.0;
	elseif i==1 & j>1 & j<nsegs 
          LAP(indm,indl)=1./3;
          LAP(indm+1,indl+1)=1./3;
          LAP(indm+2,indl+2)=1./3;
	elseif i==1 & j==nsegs
          LAP(indm,indl)=0.5;
          LAP(indm+1,indl+1)=0.5;
          LAP(indm+2,indl+2)=0.5;
        elseif j==1 & i>1 & i<ndeps
          LAP(indm,indl)=0.0;
          LAP(indm+1,indl+1)=0.0;
          LAP(indm+2,indl+2)=0.0;
        elseif j==1 & i==ndeps
          LAP(indm,indl)=0.0;
          LAP(indm+1,indl+1)=0.0;
          LAP(indm+2,indl+2)=0.0;
	elseif i==ndeps & j>1 & j<nsegs
	  LAP(indm,indl)=1./3;
          LAP(indm+1,indl+1)=1./3;
          LAP(indm+2,indl+2)=1./3;
	elseif i==ndeps & j==nsegs
	  LAP(indm,indl)=0.5;
          LAP(indm+1,indl+1)=0.5;
          LAP(indm+2,indl+2)=0.5;
	elseif j==nsegs & i>1 & i<ndeps
	  LAP(indm,indl)=1./3;
          LAP(indm+1,indl+1)=1./3;
          LAP(indm+2,indl+2)=1./3;
	else
	  LAP(indm,indl)=0.25;
          LAP(indm+1,indl+1)=0.25;
          LAP(indm+2,indl+2)=0.25;
	end
      elseif m == indr
	if i==1 & j==1
          LAP(indm,indr)=0.5;
          LAP(indm+1,indr+1)=0.5;
          LAP(indm+2,indr+2)=0.5;
	elseif i==1 & j>1 & j<nsegs 
          LAP(indm,indr)=1./3;
          LAP(indm+1,indr+1)=1./3;
          LAP(indm+2,indr+2)=1./3;
	elseif i==1 & j==nsegs
          LAP(indm,indr)=0.0;
          LAP(indm+1,indr+1)=0.0;
          LAP(indm+2,indr+2)=0.0;
        elseif j==1 & i>1 & i<ndeps
          LAP(indm,indr)=1./3;
          LAP(indm+1,indr+1)=1./3;
          LAP(indm+2,indr+2)=1./3;
        elseif j==1 & i==ndeps
          LAP(indm,indr)=0.5;
          LAP(indm+1,indr+1)=0.5;
          LAP(indm+2,indr+2)=0.5;
	elseif i==ndeps & j>1 & j<nsegs
	  LAP(indm,indr)=1./3;
          LAP(indm+1,indr+1)=1./3;
          LAP(indm+2,indr+2)=1./3;
	elseif i==ndeps & j==nsegs
	  LAP(indm,indr)=0.0;
          LAP(indm+1,indr+1)=0.0;
          LAP(indm+2,indr+2)=0.0;
	elseif j==nsegs & i>1 & i<ndeps
	  LAP(indm,indr)=0.0;
          LAP(indm+1,indr+1)=0.0;
          LAP(indm+2,indr+2)=0.0;
	else
	  LAP(indm,indr)=0.25;
          LAP(indm+1,indr+1)=0.25;
          LAP(indm+2,indr+2)=0.25;
	end
      end
    end
  end
end
