%SlipEst.m
%  Script to Estimate Slip on Dislocations and Volume changes for volcanic
%  sources for Fixed Fault Geometry.
%  Paul Segall and Roland Burgmann 1995


%Poisson's ratio
        nu = 0.25;

%Update fault geometry and station coordinates in local coords
        UpdateGeom_orig

% CLEAR PLOT FILES
	!/usr/bin/rm temp.gmtlin
%	!/usr/bin/rm volums.gmtlin
	!/usr/bin/rm obs.gmtvec
	!/usr/bin/rm mod.gmtvec
	!/usr/bin/rm obs_vert.gmtvec
	!/usr/bin/rm mod_vert.gmtvec
	!/usr/bin/rm res.gmtvec

% CREATE DESIGN MATRIX

% GPS DATA
if length(d_gps) > 0
        xy_gps = llh2localxy(llh_gps, origin);

	if nf > 0
        	G_dis = MakeGgps(dis_geom, xy_gps, nu);
	else
		G_dis = [];
	end
	if nv > 0
		G_vol = MakeGvolgps(vol_geom, xy_gps, nu);
	else
		G_vol = [];
	end


	G = [G_dis, G_vol];
else
	G = [];
end 

% LEVELING DATA

if length(d_lev) > 0
        xy_lev = llh2localxy(llh_lev, origin);

	if nf > 0
		clear G_dis;
        	G_dis = MakeGlev(dis_geom, xy_lev, nu);
	else
		G_dis = [];
	end
	if nv > 0
		clear G_vol;
		G_vol = MakeGvollev(vol_geom, xy_lev, nu);
	else
		G_vol = [];
	end


	G = [G; G_dis, G_vol];

end 
% TILT DATA
%if length(d_tilt) > 0
%        xy_tilt = llh2localxy(llh_tilt, origin);
% 
%        if nf > 0
%                clear G_dis;
%                G_dis = MakeGtilt(dis_geom, xy_tilt, nu);
%        else
%                G_dis = [];
%        end
%        if nv > 0
%                clear G_vol;
%                G_vol = MakeGvoltilt(vol_geom, xy_tilt, nu);
%        else
%                G_vol = [];
%        end
% 
% 
%        G = [G; G_dis, G_vol];
%end
% 
 
% invert and estimate slip
	Ginv = pinv(W*G);
	slip = Ginv*W*d;
	cov_slip = Ginv*Ginv';
	sig_slip = sqrt(diag(cov_slip));
	dhat = G*slip;
        r = W*(d - dhat);


% Output results
	PrintSlip(slip, sig_slip, nf, nv);
%	PrintResids(r, d_gps, d_lev, d_tilt,  G);
	PrintResids(r, d_gps, G);

% Write out volume change if opening is involved
for i=1:nf;
	if dis_geom(i,10)~=0
		clear volchg
		volchg(i) = dis_geom(i,1) * dis_geom(i,2) * slip(i*3)/1000;
		disp(['   Fault Number ', num2str(i)]);
		fprintf(1,'Volume change %8.4f',  volchg(i) ); disp(' (km^3)'); disp('  ') 
	end
end


% IF horizontal data exists -- plot  vectors

if length(d_gps) > 0
	%plot observed vectors

% This is for a 3D covariance Matrix!
%        [a,b,az] = ErrlpsGMT( cov3d2cov2d(cov_gps)  );
%        nvec = length(d_gps)/3;
 
% This is for a 2D covariance Matrix!
        [a,b,az] = ErrlpsGMT(cov_gps);
        nvec = length(d_gps)/2;
        outmatrix1 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)', vel(1,1:nvec)', ...
                vel(2,1:nvec)', a',b',az'];
%        outmatrix1v = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)', zeros(1,nvec)', vel(3,1:nvec)', zeros(1,nvec)', zeros(1,nvec)', zeros(1,nvec)'];
 
        % plot predicted vectors 
% This is for 3D 
%        u_pre = zeros(3,nvec);  
%        for i = 1:nvec
%                u_pre(:,i)  = dhat( 3*(i-1)+1 : 3*(i-1)+3 );
%        end
% This is for 2D
        u_pre = zeros(2,nvec);
        for i = 1:nvec
                u_pre(:,i)  = dhat( 2*(i-1)+1 : 2*(i-1)+2 );
        end
        null = zeros(size(u_pre(1,:)))';
        outmatrix2 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)',  u_pre(1,:)', ...
                  u_pre(2,:)', null, null, null];
%        outmatrix2v = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)', zeros(1,nvec)', u_pre(3,:)', zeros(1,nvec)', zeros(1,nvec)', zeros(1,nvec)'];
%        outmatrix2v = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)', null, u_pre(3,:)', null, null, null];
 
% plot residual vectors  
        res =   (vel(:,1:nvec)) - u_pre(:,:) ;
 
        outmatrix3 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)',  res(1,:)',   res(2,:)', null,null,null];
 
        save obs.gmtvec outmatrix1 -ascii
        save mod.gmtvec outmatrix2 -ascii
%        save obs_vert.gmtvec outmatrix1v -ascii
%        save mod_vert.gmtvec outmatrix2v -ascii
        save res.gmtvec outmatrix3 -ascii
end
 
% IF TILT DATA EXISTS -- PLOT TILT DATA IN GMT

%if length(d_tilt) > 0
%        clear outmatrix1
%        clear outmatrix2
%	clear nvec
%	clear a
%	clear b
%	clear az
%        nsta = length(d_tilt)/2
%%plot observed vectors
%        a(1) = sqrt(cov_tilt(1,1));
%        b(1) = sqrt(cov_tilt(2,2));
%        az(1) = 0;
%for i = 4:2:(nsta*2)
%        a(i/2) = sqrt(cov_tilt(i-1,i-1));
%        b(i/2) = sqrt(cov_tilt(i,i));
%        az(i/2) = 0;
%end
%        nvec = length(d_tilt_v);
%        outmatrix1 = [llh_tilt(2,1:nvec)', llh_tilt(1,1:nvec)', d_tilt_v(1,:)', ...
%                d_tilt_v(2,:)',a',b',az'];
% 
%        % plot predicted vectors
%        u_pre = zeros(2,nvec);
%        i_tilt = length(d_gps)+length(d_lev) +1
%        dhat_tilt = dhat(length(d_gps)+length(d_lev)+1:length(dhat));
%for i = 1: nvec
%                u_pre(:,i )  = dhat_tilt( 2*(i-1)+1 : 2*(i-1)+2 );
%end
%        null = zeros(size(u_pre(1,:)))';
%        outmatrix2 = [llh_tilt(2,1:nvec)', llh_tilt(1,1:nvec)', u_pre(1,:)', ...
%                 u_pre(2,:)', null, null, null];
% 
%        save tiltobs.gmtvec outmatrix1 -ascii
%        save tiltmod.gmtvec outmatrix2 -ascii
%end
%
%%IF LEVELING DATA EXISTS, PLOT GMT FILES
if length(d_lev) > 0
% GMT PLOTS
%        outmatrix4 = [llh_lev(2,:)', llh_lev(1,:)', [0;d_lev]];
%        dhat_lev = dhat(length(d_gps)+1:length(d_gps)+length(d_lev));
%        outmatrix5 = [llh_lev(2,:)', llh_lev(1,:)', [0;dhat_lev]];
%        res_lev = d_lev - dhat_lev;
%        outmatrix6 = [llh_lev(2,:)', llh_lev(1,:)', [0;res_lev]];
 
        outmatrix4 = [llh_lev(2,:)', llh_lev(1,:)', d_lev];
        dhat_lev = dhat(length(d_gps)+1:length(d_gps)+length(d_lev));
        outmatrix5 = [llh_lev(2,:)', llh_lev(1,:)', dhat_lev];
        res_lev = d_lev - dhat_lev;
        outmatrix6 = [llh_lev(2,:)', llh_lev(1,:)', res_lev];
 
        save levelobs.gmtvec outmatrix4 -ascii
        save levelmod.gmtvec outmatrix5 -ascii
        save levelres.gmtvec outmatrix6 -ascii
 
end 
 
% PRINT OUT MOMENT AND MOMENT MAGNITUDE
                clear Mo
                clear Mw
mmslip = slip/1000
for i=1:nf;

                Mo(i) = 1000 * dis_geom(i,1) * 1000 * dis_geom(i,2) * abs(sqrt(mmslip(i*3-2)^2 + mmslip(i*3-1)^2)) * 30E9;
                Mw(i) = 2/3*log10(Mo(i)) - 6.0;
        end
                Mo_total = sum(Mo);
                Mw_total = 2/3*log10(Mo_total) - 6.0;
                disp(' _______________________ ');
                fprintf(1,'Moment %4.2e',  Mo_total ); disp(' (Nm)'); disp('  ')
                fprintf(1,'Mw %8.4f',  Mw_total ); disp(' Geodetic Moment Magnitude ')
                disp(' _______________________ ');

% END PRINT OUT MOMENT AND MOMENT MAGNITUDE

% Plot Fault Outlines and Volume Sources in GMT plot

	if nf > 0
		for i=1:nf
	  	 displot_gmt( geom(i,:), dis_geom(i,:) ) 
		end
	end
	
	if nv >0
	 	 volplot_gmt( vgeom ); 
	end

% Run Script
	if get(cb_plt, 'value') == 1
		!turkey.map
	end


%IF LEVELING DATA EXISTS, PLOT PROFILES

% PROFILES

if length(d_lev) > 0
        nlevsta = length(d_lev)+1
        % plot observed level data
        figure
%        errorbar (Len,d_lev, sigma_lev, sigma_lev, 'y+')
        errorbar (llh_lev(3,:)',d_lev, sigma_lev, sigma_lev, 'y+')
        hold on

        % plot modeled level data
%        plot (Len,dhat(length(d_gps)+1:length(d_gps)+length(d_lev)), 'g*')
        plot (llh_lev(3,:)',dhat(length(d_gps)+1:length(d_gps)+length(d_lev)), 'g*')
        hold off;
end 
%
%

%% Plot Geometry in Local Cartesian Coordinates
%        figure
%if length(d_gps) > 0
%        plot(xy_gps(:,1), xy_gps(:,2), 'o'), axis('equal'), hold on
%end
%if length(d_lev) > 0
%        plot(xy_lev(:,1), xy_lev(:,2), 'x'), axis('equal'), hold on
%end
%        if length(d_tilt) > 0
%                plot(xy_tilt(:,1), xy_tilt(:,2), '+'), axis('equal'), hold on
%        end
%
%        for i=1:nf
%         displot(dis_geom(i,:))
%        end
%       hold off
%%%
%% Quiver and uplift plot of velocity field
%
%% Plotting range
%
%xmin = -60; xmax = 60; ymin = -60; ymax = 60;
%%
%% receiver geometry
%y = linspace(ymin, ymax,  50);
%x = linspace(xmin, xmax,  50);
%n = length(x);  m = length(y);
%
%% initialize plot arrays to zero
%uz = zeros(n,m); ux = zeros(n,m);  uy=zeros(n,m);
%%
%for i= 1:m,
%  for j=1:n,
%     for k=1:nf
%     u = disloc(nu, dis_geom(k,:),[x(j),y(i)]);
%     uz(i,j)  = uz(i,j) + u(3);
%     ux(i,j)  = ux(i,j) + u(1);
%     uy(i,j)  = uy(i,j) + u(2);
%     end
%  end
%end
%%%
%hold on
%C = contour(x, y, uz, 10);
%clabel(C);
%title('1975 Kilauea velocity field and uplift contours')
%hold on
%quiver(x,y,ux,uy)
%hold off
%%%
%%%
% 
disp('Done')
 
