%SlipEst.m
% Modified March 10, 1999 to include SAR data.
% Modified November 10, 1999 to include Laplacian filter --EJP.
%  Script to Estimate Slip on Dislocations and Volume changes for volcanic
%  sources for Fixed Fault Geometry.
%  Paul Segall and Roland Burgmann 1995


%Poisson's ratio
        nu = 0.25;

%Update fault geometry


%Set the model origin and convert station coordinates to local coords
  SetOrigin

% DISCRETIZE A FAULT
%nsegs = input('Number of along strike (length) segments:  ');
%ndeps = input('Number of down dip (width) segments:       ');

tmp_geom = []
   for k = 1:nf
        [tmp_geom] = [tmp_geom; patchfault(dis_geom(1,:),nsegs,ndeps)];
   end
fault_slip = (dis_geom(1,8:10)'*ones(1,nf*nsegs*ndeps))';
fault_pgeom = [tmp_geom, fault_slip];
dis_geom = sortrows(fault_pgeom, 3);

disgeom2geom

% CREATE DESIGN MATRIX

% GPS DATA
if length(d_gps) > 0
        xy_gps = llh2localxy(llh_gps, origin);

	if nf > 0
        	G_dis = MakeGgps(dis_geom, xy_gps, nu);
	else
		G_dis = [];
	end
	if nv > 0
		G_vol = MakeGvolgps(vol_geom, xy_gps, nu);
	else
		G_vol = [];
	end


	G = [G_dis, G_vol];
else
	G = [];
end 


% LEVELING DATA

if length(d_lev) > 0
        xy_lev = llh2localxy(llh_lev, origin);

	if nf > 0
		clear G_dis;
        	G_dis = MakeGlev(dis_geom, xy_lev, nu);
	else
		G_dis = [];
	end
	if nv > 0
		clear G_vol;
		G_vol = MakeGvollev(vol_geom, xy_lev, nu);
	else
		G_vol = [];
	end


	G = [G; G_dis, G_vol];

end 

% SAR DATA

if length(d_SAR) > 0
        xy_SAR = llh2localxy(llh_SAR, origin);

	if nf > 0
		clear G_dis;
        	G_dis = MakeGSAR(dis_geom, xy_SAR, nu, origin);
	else
		G_dis = [];
	end
%	if nv > 0
%		clear G_vol;
%		G_vol = MakeGvolSAR(vol_geom, xy_SAR, nu);
%	else
%		G_vol = [];
%	end


%	G = [G; G_dis, G_vol];
%	G = [G; G_dis];

else
	G_dis=[];
end 
% Smoothing constraints here.
% Make sure nsegs and ndeps are defined.

	G_lap=MakeLapF(nsegs,ndeps);

% Scale the smoothing matrix
% For coseismic we use 25 For GPS alone
% For coseismic we use 25 For InSAR alone
% For coseismic we use 25 For GPS and InSAR

	G_lap = 2.0 * G_lap;
        d_lap=zeros(3*nf,1);

% FOR DISCRETIZED FAULT
%        d_lap=zeros(3*nf*nsegs*ndeps,1);

% invert and estimate slip
	len_geod=length(d_lev)+length(d_gps);
%       TO ADD WEIGHTING TO THE SMOOTHING:
	if length(d_SAR) > 0
%       TO ADD WEIGHTING TO THE SAR DATA:
	W_SAR = 30
          if len_geod > 0
 	    G2I=[W*G zeros(len_geod,3);W_SAR*G_dis;G_lap zeros(nsegs*ndeps*3,3)];
          else
  	    G2I=[W_SAR*G_dis;G_lap zeros(nsegs*ndeps*3,3)];
	  end
	else
  	  G2I=[W*G;G_dis;G_lap];
	end
% FOR FULL COVARIANCE MATRIX AND NO SMOOTHING
%	Ginv = pinv(W*G);
%	slip = Ginv*W*d;
%        cov_slip = Ginv*Ginv';
%        sig_slip = sqrt(diag(cov_slip));
%        dhat = G*slip;
%        r = W*(d - dhat);
% END ORIGINAL LEAST SQUARES

% FOR SMOOTHED SLIP ESTIMATE
	Ginv = pinv(G2I);
	d2I=[W*d(1:len_geod);W_SAR*d_SAR(:);d_lap];
	slip = Ginv*d2I;
        disp('   WITH LAPLACIAN SMOOTHING ');
% END SMOOTHED

% FOR UNSMOOTHED SLIP ESTIMATE
%	G2I=[W*G zeros(len_geod,3);G_dis];
%	Ginv = pinv(G2I);
%	d2I=[W*d(1:len_geod);d_SAR(:)];
%	slip = Ginv*d2I;
%       disp('   WITHOUT LAPLACIAN SMOOTHING ');
% END UNSMOOTHED

% PRINT SLIP
	cov_slip = Ginv*Ginv';
	sig_slip = sqrt(diag(cov_slip));

%	PrintSlip(slip(1:nf*3), sig_slip(1:nf*3), nf, nv);


nfaults=nsegs*ndeps;
for k=1:nfaults,
  ss_slip(k)=slip(3*(k-1)+1);
  ss_sig_slip(k)=sig_slip(3*(k-1)+1);
  dip_slip(k)=slip(3*(k-1)+2);
  opening(k)=slip(3*(k-1)+3);
end

if length(d_SAR) > 0
ss_slip = ss_slip(1:length(slip)/3-1);
end

mapslip3d(dis_geom, ss_slip);
mapslip3d(dis_geom, dip_slip);

% END PRINT SLIP

% SLIP DISTRIBUTION WITH BOUNDS
% SET UP THE BOUNDS
 bounds_bvls
% RUN BVLS
 [slipb,wb,istateb,loopA]=bvls(G2I,d2I,bl,bu,slip,istate);


% COMPUTE RESOLUTION MATRICES (OFF NOW) and SIGMAS OF BVLS SLIP 
 bvls_compute_resolu
 slipb=slipb';
for k=1:nfaults,
  ss_slipb(k)=slipb(3*(k-1)+1);
  dip_slipb(k)=slipb(3*(k-1)+2);
  openingb(k)=slipb(3*(k-1)+3);
  ss_sig_slipb(k)=sig_slipb(3*(k-1)+1);
  dip_sig_slipb(k)=sig_slipb(3*(k-1)+2);
  sig_openingb(k)=sig_slipb(3*(k-1)+3);
end

% PLOT THE BVLS DISTRIBUTED SLIP MODEL
 mapslip3d(dis_geom, ss_slipb);
 mapslip3d(dis_geom, dip_slipb);

% MAKE A PLOT OF SURFACE SLIP
% FOR DISCRETIZED FAULT
patch = [geom(1:nf,5)]; 
for i=1:nsegs-1
patch = [patch, geom(1:nf,5) + i * (geom(1:nf,7) - geom (1:nf,5))/(nsegs-1)];
end
%patch = []
%patch = [patch,  (geom(1:nsegs,5) + geom (1:nsegs,7))/2];
ss_surface_slip = [patch; ss_slipb(1:nsegs);  ss_sig_slipb(1:nsegs)]';
ds_surface_slip = [patch;dip_slipb(1:nsegs); dip_sig_slipb(1:nsegs)]';
save  surface_slip_ss.txt  ss_surface_slip -ascii
save  surface_slip_dip.txt ds_surface_slip -ascii
!surface_slip.map

%PLOT  THE RESOLUTION MATRIX
% resol = diag(R);
%for k=1:nfaults,
% resol_ss(k) = resol(3*(k-1)+1);
%end
%mapslip3d(dis_geom, resol_ss);
% WITH BVLS
% resol_bvls = diag(Rb);
%mapslip3d(dis_geom, resol_bvls, [],[],0);

% COMPUTE AND PRINT RESIDUALS

if length(d_SAR) > 0
	dhat = [G zeros(len_geod,3);G_dis]*slipb;
	d=[d;d_SAR];
        len_all=len_geod+length(d_SAR(:));
        r(len_geod+1:len_all)=d(1+len_geod:len_all) - dhat(1+len_geod:len_all);
	r_SAR = r(len_geod+1:len_all)';
	n_SAR = length(d_SAR)
        rank_SAR = rank(G_dis);
        disp('GOODNESS OF FIT:  SAR')
        fprintf('SAR:   Weighted Residual Sum of Squares (WRSS), %g\n', W_SAR*r_SAR'*W_SAR*r_SAR)
        fprintf('SAR:   WRSS / (N-P),         %g\n', (W_SAR*r_SAR'*W_SAR*r_SAR)/(n_SAR-rank_SAR) )
        fprintf('SAR:   WRSS / (N),         %g\n', (W_SAR*r_SAR'*W_SAR*r_SAR)/(n_SAR) )
        disp('__________________________________ ')

else
	  dhat = G*slipb;
end

if(len_geod > 0)
 	  r(1:len_geod) = W*(d(1:len_geod) - dhat(1:len_geod));
        disp('__________________________________ ')
        disp(' ')

        n_gps = length(d_gps);
	r_gps = r(1:len_geod)';
        rank_gps = rank(G(1:n_gps,:));
        disp('GOODNESS OF FIT:  GPS')
        fprintf('GPS:   Weighted Residual Sum of Squares (WRSS), %g\n', r_gps'*r_gps)
        fprintf('GPS:   WRSS / (N-P),         %g\n', (r_gps'*r_gps)/(n_gps-rank_gps) )
        fprintf('GPS:   WRSS / (N),         %g\n', (r_gps'*r_gps)/(n_gps) )
        disp('__________________________________ ')
        disp(' ')
        disp(' ')
end

%	  PrintResids(r_gps, d_gps, G);
%	end
	
% END: COMPUTE AND PRINT RESIDUALS

% SUBTRACT A RAMP FROM RESIDUALS?
%I would subtract the model from the data and fit a plane to the residuals.
%Then you could subtract this plane from the data and look at the sum of
%the squares of the differences between the detrended data and the model.
 
%if length(d_SAR) > 0
%	Gplane = [xy_SAR ones(size(xy_SAR(:,2)))];
%	m=pinv(Gplane)*r_SAR;
%	plane=Gplane*m;
%	dnew=d_SAR-plane;
%end

%% Clear Plot Files
	!/usr/bin/rm temp.gmtlin
	!/usr/bin/rm obs.gmtvec
	!/usr/bin/rm mod.gmtvec
        !/usr/bin/rm obs_vert.gmtvec
        !/usr/bin/rm mod_vert.gmtvec
        !/usr/bin/rm res.gmtvec
        !/usr/bin/rm sarmod.xyz
        !/usr/bin/rm sarobs.xyz
        !/usr/bin/rm sarres.xyz
 

% GPS DATA
 
if length(d_gps) > 0

% This is for a 3D covariance Matrix!
        [a,b,az] = ErrlpsGMT( cov3d2cov2d(cov_gps)  );
        nvec = length(d_gps)/3;

% This is for a 2D covariance Matrix!
%        [a,b,az] = ErrlpsGMT(cov_gps);
%        nvec = length(d_gps)/2;

        outmatrix1 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)', vel(1,1:nvec)', ...
                vel(2,1:nvec)', a',b',az'];
        outmatrix1v = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)', zeros(1,nvec)', ...
                vel(3,1:nvec)', zeros(1,nvec)', zeros(1,nvec)', zeros(1,nvec)'];

        % plot predicted vectors
% This is for 3D
        u_pre = zeros(3,nvec);
        for i = 1:nvec
                u_pre(:,i)  = dhat( 3*(i-1)+1 : 3*(i-1)+3 );
        end
% This is for 2D
%        u_pre = zeros(2,nvec);
%        for i = 1:nvec
%                u_pre(:,i)  = dhat( 2*(i-1)+1 : 2*(i-1)+2 );
%        end

        null = zeros(size(u_pre(1,:)))';
        outmatrix2 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)',  u_pre(1,:)', ...
                  u_pre(2,:)', null, null, null];
        outmatrix2v = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)', zeros(1,nvec)', ...
                  u_pre(3,:)', zeros(1,nvec)', zeros(1,nvec)', zeros(1,nvec)'];

% plot residual vectors
        res =   (vel(:,1:nvec)) - u_pre(:,:) ;

        outmatrix3 = [llh_gps(2,1:nvec)', llh_gps(1,1:nvec)',  res(1,:)',   res(2,:)', null,null,null];

        save obs.gmtvec outmatrix1 -ascii
        save mod.gmtvec outmatrix2 -ascii
        save obs_vert.gmtvec outmatrix1v -ascii
        save mod_vert.gmtvec outmatrix2v -ascii
        save res.gmtvec outmatrix3 -ascii
end
% END GPS DATA

% INSAR DATA
if length(d_SAR) > 0
	outmatrix3 =  [llh_SAR(2,:)', llh_SAR(1,:)', dhat(1+len_geod:len_all)];
	outmatrix4 =  [llh_SAR(2,:)', llh_SAR(1,:)', r_SAR];
	outmatrix5 =  [llh_SAR(2,:)', llh_SAR(1,:)', d(1+len_geod:len_all)];
	save sarmod.xyz outmatrix3 -ascii
	save sarres.xyz outmatrix4 -ascii
	save sarobs.xyz outmatrix5 -ascii
end
% END INSAR DATA

% END COMPUTE AND PRINT RESIDUALS

% PLOT FAULT OUTLINES AND VOLUME SOURCES IN GMT PLOT

	if nf > 0
		for i=1:nf
	  	 displot_gmt( geom(i,:), dis_geom(i,:) ) 
		end
	end
	
	if nv >0
	 	 volplot_gmt( vgeom ); 
	end
%
% PRINT OUT MOMENT AND MOMENT MAGNITUDE
                clear Mo
                clear Mw
for i=1:nf;

                Mo(i) = 1000 * dis_geom(i,1) * 1000 * dis_geom(i,2) * abs(sqrt(slip(i*3-2)^2 + slip(i*3-1)^2)) * 30E9;
                Mw(i) = 2/3*log10(Mo(i)) - 6.0;
                Mo_b(i) = 1000 * dis_geom(i,1) * 1000 * dis_geom(i,2) * abs(sqrt(slipb(i*3-2)^2 + slipb(i*3-1)^2)) * 30E9;
                Mw_b(i) = 2/3*log10(Mo(i)) - 6.0;
%                disp(['   Fault Number ', num2str(i)]);
%                fprintf(1,'Moment %4.2e',  Mo(i) ); disp(' (Nm)'); disp('  ')
%                fprintf(1,'Mw %8.4f',  Mw(i) ); disp('  ')
        end
		Mo_total = sum(Mo);
                Mw_total = 2/3*log10(Mo_total) - 6.0;
		Mo_b_total = sum(Mo_b);
                Mw_b_total = 2/3*log10(Mo_b_total) - 6.0;
                disp(' _______________________ ');
                fprintf(1,'Moment %4.2e',  Mo_total ); disp(' (Nm)'); disp('  ')
                fprintf(1,'Mw %8.4f',  Mw_total ); disp(' Geodetic Moment Magnitude ')
                disp(' _______________________ ');
                fprintf(1,'Moment with positivity %4.2e',  Mo_b_total ); disp(' (Nm)'); disp('  ')
                fprintf(1,'Mw %8.4f',  Mw_b_total ); disp(' Geodetic Moment Magnitude with positivity ')
%		Mo_post = sum(Mo)*75/365;
%                Mw_post = 2/3*log10(Mo_post) - 6.0;
%                disp('   Postseismic (75 days) Geodetic Moment ');
%                disp(' _______________________ ');
%                fprintf(1,'Moment %4.2e',  Mo_post ); disp(' (Nm) postseismic slip'); disp('  ')
%                fprintf(1,'Mw %8.4f',  Mw_post ); disp(' Geodetic Moment Magnitude ')

% END PRINT OUT MOMENT AND MOMENT MAGNITUDE

% RUN GMT SCRIPT
	if get(cb_plt, 'value') == 1
		!test.map
	end
% END RUN GMT SCRIPT
%
%
%% IF Leveling data exists plot observed and predicted
%
%if length(d_lev) > 0
% 
%	% plot observed level data
%        figure
%        errorbar (Len,d_lev, sigma_lev, sigma_lev, 'y+')
%        hold on
% 
%	% plot modeled level data
%        plot (Len,dhat(length(d_gps)+1:length(d_gps)+length(d_lev)), 'g')
%        hold off;
%end
%
disp('Done')
