function [] = mapslip3d(pm,slip,varargin)
%
% usage: mapslip3d(fault_geom, slip_vector, [linefile, overlay, map_z])
%
% function plots variable slip estimates onto a 3-D plot.
%
% INPUT
% fault_geom  - n X 7 matrix of fault patches (same format as dis_geom in DisModel)
% slip_vector - 1 X n vector of estimated slip values  
%
% (optional inputs)
% linefile    - string, name of .mat file that contains xy-values to be plotted.
%		.mat file must have xy-values saved in cell array called xylines.
% overlay     - 'o', if plot should be overlaid rather than a new figure
% map_z       - scalar, z value at which the linefile should be plotted
%
% 
numvargin = length(varargin);

if numvargin > 0
	if ~isempty(varargin{1})
		load(varargin{1}(:));
	else
	    xylines = [];
	end
end

if numvargin > 1
	if varargin{2} == 'o'
		hold on
	else
		figure
	end
else
	figure
end

if numvargin > 2
	map_z = varargin{3};
end
	
% set fault parameters 
wid = pm(1,2);
len = pm(1,1);
i = sqrt(-1);
dipr = pm(1,4)*pi/180;
east = pm(:,6);
north = pm(:,7);
up = -1*pm(:,3);

% corners in local coordinates
ll = -0.5*len; lr =0.5*len;
ul = -0.5*len + i*wid*cos(dipr); ur = 0.5*len + i*wid*cos(dipr);
%
% z-component
bottom = up;
top = up + wid*sin(dipr);
%
% transform corners
%strkr = (90-pm(1,5))*pi/180;
% FIX ON 04/09 by SUE
strkr = (90-pm(:,5))*pi/180;


ll = (east+north*i) + ll*exp(i*strkr);
lr = (east+north*i) + lr*exp(i*strkr);
ul = (east+north*i) + ul*exp(i*strkr);
ur = (east+north*i) + ur*exp(i*strkr);

xvert = [real(lr)'; real(ur)'; real(ul)'; real(ll)'];
yvert = [imag(lr)'; imag(ur)'; imag(ul)'; imag(ll)'];
zvert = [bottom'; top';top';bottom'];

%
% plot the slip values

patch(xvert,yvert,zvert,slip,'EdgeColor',[1 1 1]), axis('equal')
colormap('jet')
colorbar
hold on

if ~isempty(xylines)
for i=1:length(xylines)
	z = map_z*ones(size(xylines{i}(:,1))); 
	plot3(xylines{i}(:,1),xylines{i}(:,2),z,'k');	
end
end
%view([5, 40])
view([8, 18])

xlabel('Distance W to E (km)')
zlabel('Depth (km)')
axis([-2   52   -10    10   -18     0])
%axis([-30   30   -10    10   -15     0])
%text (0, -10, 0, 'Slip Distributed Model')

